package dybip.system;

import java.util.ArrayList;

import dybip.Context;
import dybip.MotifInstanceCountMap;
import dybip.ParameterMap;
import dybip.InteractionStmt.EnabledInterReconfiguration;
import dybip.InteractionStmt.EnabledInteraction;
import dybip.InteractionStmt.EnabledReconfiguration;
import dybip.InteractionStmt.EnabledStepList;
import dybip.InteractionStmt.ExecutableInterReconfigurationRule;
import dybip.InteractionStmt.ExecutableInterReconfigurationRuleList;
import dybip.InteractionStmt.InterReconfigurationRuleStmtList;
import dybip.componentInstance.ComponentInstance;
import dybip.componentInstance.IntegerData;
import dybip.componentType.ComponentTypeMap;
import dybip.expression.Expression;
import dybip.expression.literal.IntegerLiteral;
import dybip.motifInstance.Group;
import dybip.motifInstance.MotifInstance;
import dybip.motifInstance.MotifInstanceMap;
import dybip.motifInstance.MotifTypeInstanceMap;
import dybip.motifType.MotifTypeMap;
import dybip.systemStmt.motifInstanceStmt.MotifInstanceStmtList;

public class SystemContext extends Context{
	private MotifTypeInstanceMap mtimap;
	// rules at system level
	private ExecutableInterReconfigurationRuleList systemInterReconfigRules;
	// list of enabled steps at system level
	private	EnabledStepList<EnabledInterReconfiguration> enabledSystemReconfigList;

	private MotifTypeMap mtmap;
	private ComponentTypeMap ctmap;

	public SystemContext(MotifInstanceStmtList milist, MotifInstanceCountMap micmap, ParameterMap pmap, MotifTypeMap mtmap, ComponentTypeMap ctmap, InterReconfigurationRuleStmtList interReconfigRuleStmt) {
		if(milist.size() == 0) // no instances defined in code
		{
			addParameters(pmap);
			mtimap = new MotifTypeInstanceMap(milist, micmap, mtmap, ctmap, this);
		}
		else
			mtimap = new MotifTypeInstanceMap(milist, mtmap, ctmap, getDataMap());
		this.mtmap = mtmap;
		systemInterReconfigRules = new ExecutableInterReconfigurationRuleList(interReconfigRuleStmt, this);
		enabledSystemReconfigList = new EnabledStepList<EnabledInterReconfiguration>();
		this.ctmap = ctmap;
	}

	@Override
	public Expression find(String name) {
		if(getMotifArrayListMap().containsKey(name)){
			ArrayList<MotifInstance> motifList = getMotifArrayListMap().get(name);
			int index = getIndexMap().get(name).value(Integer.class);
			return motifList.get(index);
		}
		else if(getDataMap().containsKey(name))
			return getDataMap().get(name);
		else if(mtmap != null & mtmap.containsKey(name))
			return mtmap.get(name);
		else if(getIndexMap().containsKey(name))
			return getIndexMap().get(name);
		return null; 
	}
	private void addParameters(ParameterMap pmap) {
		for(String pramName: pmap.keySet()){
			IntegerData intData = new IntegerData(pramName);
			intData.setData(new IntegerLiteral(pmap.get(pramName).getValue()));
			getDataMap().put(pramName, intData);
		}
	}
	public String getTrace() {
		return mtimap.getTrace();
	}

	/*
	 * Evalauate enabled steps
	 */
	public void evaluateEnabledSteps(){
		resetEnabledSteps();
		evaluateEnabledStepsInMotifs();
		evaluateEnabledStepsInSystem();
	}
	public void evaluateEnabledStepsInSystem(){
		// TODO updated enabled interactions and enabled reconfigurations list using the executable version of the sterp rules
		for(int i = 0; i < systemInterReconfigRules.size(); i++) {
			ExecutableInterReconfigurationRule rule = systemInterReconfigRules.get(i);
			rule.evaluateEnabledInterReconfiguration(this);
			enabledSystemReconfigList.add(rule.getEnabledSteps());
		}
	}
	public void evaluateEnabledStepsInMotifs(){
		// evaluate enabled interactions in each motif and update motif enabled steps list	
		mtimap.evaluateEnabledStepsInMotifs(); 
	}
	public void resetEnabledSteps(){	
		enabledSystemReconfigList = new EnabledStepList<EnabledInterReconfiguration>();
	}

	/*
	 * Execute Step
	 */
	public void Execute(String motifInstanceName, String stepType, int stepIndex) {
		MotifInstance motifInstance = mtimap.getMotifInstance(motifInstanceName);
		motifInstance.execute(stepType, stepIndex);
	}
	public void Execute(int stepIndex) {
		enabledSystemReconfigList.get(stepIndex).execute(this);
	}

	/*
	 *  Get Enabled step list
	 */
	public EnabledStepList<EnabledInterReconfiguration> getEnabledInterReconfigurationStepList(){
		return enabledSystemReconfigList;
	}
	public EnabledStepList<?> getEnabledMotifStepList(String motifInstanceName, String stepType){
		MotifInstance instance = mtimap.getMotifInstance(motifInstanceName);
		if(stepType.equals("r") || stepType.equals("R"))
			return instance.getEnabledReconfigurations();
		else if(stepType.equals("i") || stepType.equals("I"))
			return instance.getEnabledInteractions();
		else {
			System.err.println("Unknown Step Type " + stepType);
			System.exit(0);
			return null;
		}
	}


	public ComponentTypeMap getComponentTypeMap() {
		return ctmap;
	}
	public int getComponentInstanceCount() {
		return mtimap.getComponentInstanceCount();
	}
	public int getMotifInstanceCount() {
		return mtimap.getMotifInstanceCount();
	}
	public int getEnabledInteractionCount() {
		return mtimap.getEnabledInteractionCount();
	}
	public int getEnabledReconfigurationCount() {
		int motifReconfiguration = mtimap.getEnabledReconfigurationCount();
		int interMotifReconfiguration = enabledSystemReconfigList.size();
		return motifReconfiguration + interMotifReconfiguration;
	}

	public ArrayList<String> getMotifInstanceNames(){
		return mtimap.getMotifInstanceNames();
	}
	public ArrayList<MotifInstance> getMotifListOfType(String type){
		ArrayList<MotifInstance> marrayList = new ArrayList<MotifInstance>();

		MotifInstanceMap  mimap = mtimap.get(type);
		if(mimap!= null)
			for(String key: mimap.keySet()) {
				marrayList.add(mimap.get(key));
			}
		return marrayList;
	}

	/* TRACK Size
	 */

	public String trackMotifSize(String motifType, int step, int runCount) {
		String [] motifTypes = motifType.split(",");
		String sizes = "";
		for(int i = 0; i < motifTypes.length; i++)
		{	
			if(motifTypes[i].equals("Processor"))
			{
				
				MotifInstanceMap instances = mtimap.get(motifTypes[i]);
				for(String instName: instances.keySet()) {
					MotifInstance mi = instances.get(instName);
					Group g = mi.getGroup("cores");
					for(int j = 0; j < g.size(); j++)
					{
						long id = g.get(j).getID();
						// find coreTask motif with this ID
						if(mtimap.containsComponentID("ReateCoreTask", id))
							mi = mtimap.findMotifByID("ReateCoreTask", id);
						else if(mtimap.containsComponentID("CoreTask", id))
							mi = mtimap.findMotifByID("CoreTask", id);
						else if(mtimap.containsComponentID("DeleteCoreTask", id))
							mi = mtimap.findMotifByID("DeleteCoreTask", id);	
						
						sizes += mi.getGroup("tasks").size() + "\t";
					}
					return sizes + "\n";
				}
			}
			else
			{
				MotifInstanceMap instances = mtimap.get(motifTypes[i]);
				for(String instName: instances.keySet()) {
					MotifInstance mi = instances.get(instName);
					sizes += mi.trackMotifSize(step, instName, runCount);
				}
			}
		}
		return sizes;
	}
	/*
	 *  ADD/REMOVE motifs
	 */
	public void removeMotif(String type, String name) {
		mtimap.remove(type, name);
	}
	public void addMotif(MotifInstance minstance) {

		String motifType = minstance.getTypeStr();
		if(mtimap.keySet().contains(motifType)) {
			MotifInstanceMap mimap = mtimap.get(motifType);
			mimap.put(minstance.getName(), minstance);
		}
		else{
			// first instance of a motif type
			MotifInstanceMap mimap = new MotifInstanceMap(1);
			mimap.put(minstance.getName(), minstance);
			mtimap.put(motifType,mimap);
		}
	}
	/*
	 *  Has enabled Step
	 */
	public boolean hasEnabledMotifStep() {
		return hasEnabledMotifReconfiguration() || hasEnabledMotifInteraction();
	}
	public boolean hasEnabledMotifInteraction() {
		return mtimap.hasEnabledMotifInteraction();
	}
	public boolean hasEnabledMotifReconfiguration(){
		return mtimap.hasEnabledMotifReconfiguration();
	}
	public boolean hasEnabledInterReconfigurationStep() {
		return systemInterReconfigRules.hasEnabledSteps();
	}
	/*
	 *  To print methods
	 */
	public void printChosenStep(int stepIndex) {
		String log = "CHOSEN INTER-RECONFIGURATION STEP\nINDEX: " + stepIndex;
		System.out.println(log + enabledSystemReconfigList.get(stepIndex).toMinifyString(this));
	}
	public void printChosenStep(String motifInstanceName, String stepType, int stepIndex) {
		MotifInstance motifInstance = mtimap.getMotifInstance(motifInstanceName);
		System.out.println(motifInstance.chosenStepToString(stepType, stepIndex));
	}
	public void printEnabledSteps() {
		System.out.println("/********************** ENABLED STEPS*******************/\n");
		printEnabledMotifInteractions();
		printEnabledMotifReconfigurations();
		printEnabledInterReconfigurations();
	}
	private void printEnabledMotifInteractions() {
		if(mtimap.keySet().size() == 0)
			System.err.println("NO ENABLED INTERACTIONS IN MOTIFS");
		else {
			for(String type: mtimap.keySet()) {
				MotifInstanceMap mimap = mtimap.get(type);
				for(String key: mimap.keySet()){
					MotifInstance mInstance = mimap.get(key);
					EnabledStepList<EnabledInteraction> enabledInteraction = mInstance.getEnabledInteractions();

					if(enabledInteraction.size() != 0) {
						System.out.println(key + "   INTERACTION");
						System.out.println(mInstance.enabledInteractionsToString());
					}
					else
						System.err.println("NO ENABLED INTERACTIONS IN MOTIF " + key);
				}
			}
		}
	}
	private void printEnabledMotifReconfigurations() {
		if(mtimap.keySet().size()==0)
			System.err.println("NO ENBALED RECONFIGURATIONS IN MOTIFS");
		else
		{
			for(String type: mtimap.keySet())
			{
				MotifInstanceMap mimap = mtimap.get(type);
				for(String key: mimap.keySet()){

					MotifInstance mInstance = mimap.get(key);
					EnabledStepList<EnabledReconfiguration> enabledReconfigurations = mInstance.getEnabledReconfigurations();

					if(enabledReconfigurations.size() != 0) {
						System.out.println(key + "   RECONFIGURATION");
						System.out.println(mInstance.enabledReconfigurationsToString());
					}
					else
						System.err.println("NO ENABLED Reconfigurations IN MOTIF " + key);
				}
			}
		}
	}
	private void printEnabledInterReconfigurations() {
		if(enabledSystemReconfigList.size() ==0) {
			System.err.println("NO ENABLED INTER-MOTIF RECONFIGURATIONS IN MOTIFS");}
		else {
			System.out.println("   INTER-RECONFIGURATION");
			System.out.println(enabledSystemReconfigList.toString());
		}
	}

	@Override
	public String toString(){
		return mtimap.toString();
	}

}
